#!/usr/bin/env perl

use strict;
use warnings;

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use FindBin;
use lib ("$FindBin::Bin/../../Trinotate/PerlLib", "$FindBin::Bin/PerlLib");
use DBI;
use Sqlite_connect;
use CanvasXpress::Heatmap;
use Data::Dumper;

main: {
    
    my $cgi = new CGI();
    print $cgi->header();
    
    my %params = $cgi->Vars();

    print $cgi->start_html();    
    my $sqlite_db = $params{sqlite} or die "Error, need sqlite param";
        
    &write_heatmap($sqlite_db, \%params);

    print $cgi->end_html();
    
    exit(0);
}

####
sub write_heatmap {
    my ($sqlite_db, $params_href) = @_;
    
    my $dbproc = DBI->connect( "dbi:SQLite:$sqlite_db" ) || die "Cannot connect: $DBI::errstr";

    
    my $min_FC = $params_href->{min_FC} || 4;
    my $max_FDR = $params_href->{max_FDR} || 0.001;
        
    # print "<pre>\n" . Dumper($params_href) . "</pre>\n";
    
    my $html = "<div id='control_box'>\n";
    
    $html .= "<form id='control_form' method='get' action='TrinotateViewer.cgi' >\n";
    $html .= "<ul>\n";
    $html .= "   <li>min_FC: <input type='text' name='min_FC' value=\"$min_FC\" />\n";
    $html .= "   <li>max_FDR: <input type='text' name='max_FDR' value=\"$max_FDR\" />\n";
    {
        $html .= "   <li><input type='checkbox' name='cluster_transcripts' value='1' ";
        if ($params_href->{cluster_transcripts}) {
            $html .= " checked ";
        }
        $html .= " />Cluster transcripts\n";
    }
    
    {
        $html .= "   <li><input type='checkbox' name='cluster_samples' value='1' ";
        if ($params_href->{cluster_samples}) {
            $html .= " checked ";
        }
        $html .= " />Cluster samples\n"; 
    }

    $html .= "   <li><input type='submit' />\n";
    $html .= "</ul>\n";

    $html .= "<input type='hidden' name='sqlite' value=\'$sqlite_db\' />\n";
    
    $html .= "</form>\n";
    $html .= "</div>\n";

    print $html;

    
    my %sample_id_to_sample_name;
    my $query = "select sample_id, sample_name from Samples";
    my @results = &do_sql_2D($dbproc, $query);
    foreach my $result (@results) {
        my ($sample_id, $sample_name) = @$result;
        $sample_id_to_sample_name{$sample_id} = $sample_name;
    }
    
    
    my %replicate_id_to_info;
    $query = "select replicate_id, replicate_name, sample_id from Replicates";
    @results = &do_sql_2D($dbproc, $query);
    foreach my $result (@results) {
        my ($replicate_id, $replicate_name, $sample_id) = @$result;
        $replicate_id_to_info{$replicate_id} = { 
            sample_id => $sample_id,
            replicate_name => $replicate_name,
        };
    }
    
    my $min_log_FC = log($min_FC)/log(2);
    
    
    ## get list of differentially expressed transcripts
    my %trans_diff_expr;
    $query = "select distinct e.feature_name, e.replicate_id, e.fpkm "
        . " from Expression e, Diff_expression d "
        . " where e.feature_name = d.feature_name "
        . " and e.feature_type = 'T' and e.feature_type = d.feature_type "
        # add p-value, FDR, or log_fold_change cutoffs here as needed.
        . " and d.fdr <= $max_FDR"
        . " and abs(d.log_fold_change) >= $min_log_FC ";
    
    @results = &do_sql_2D($dbproc, $query);

    my %feature_to_info;
    my %replicates;
    
    foreach my $result (@results) {

       my ($feature_name, $replicate_id, $fpkm) = @$result;
       
       $feature_to_info{$feature_name}->{$replicate_id} = $fpkm;
       
       $replicates{$replicate_id} = 1;
       
    }

    ## organize data for heatmap interface
    my @replicate_ids = sort keys %replicates;
    
    my @feature_matrix;
    foreach my $feature_name (keys %feature_to_info) {
        my @row = ($feature_name);
        foreach my $replicate_id (@replicate_ids) {
            my $fpkm = $feature_to_info{$feature_name}->{$replicate_id} || 0; ## should check to be sure every one has a value here, using default for now  ## //FIXME
            
            $fpkm = log($fpkm + 1)/log(2);
            push (@row, $fpkm);
        }
        push (@feature_matrix, [@row]);
    }
    
    my @replicate_names;
    foreach my $replicate_id (@replicate_ids) {
        my $replicate_name = $replicate_id_to_info{$replicate_id}->{replicate_name};
        push (@replicate_names, $replicate_name);
    }
    
    print "<p>Found " . scalar(@feature_matrix) . " features.</p>\n";
    
    my %heatmap_inputs = (samples => [@replicate_names],
                          value_matrix => [@feature_matrix],
                          cluster_features => $params_href->{cluster_transcripts},
                          cluster_samples => $params_href->{cluster_samples},
                          
        );
    
    print &CanvasXpress::Heatmap::draw(%heatmap_inputs);

    
    return;
}
