#!/usr/bin/env perl

use strict;
use warnings;

# standard perl modules
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use FindBin;
use DBI;
use URI::Escape;
use Data::Dumper;

# custom modules
use lib ("$FindBin::Bin/../../Trinotate/PerlLib", "$FindBin::Bin/PerlLib");
use Sqlite_connect;


main: {
    
    my $cgi = new CGI();
    print $cgi->header();
    
    my %params = $cgi->Vars();
    
    if (my $sqlite_db = $params{sqlite}) {
        print $cgi->start_html();
        
        &TrinityWebMain(\%params, $sqlite_db);
        

        print $cgi->end_html();

    }
    else {
        print $cgi->start_html();
        print "Need it.\n";
        
        &print_get_sqlite_db_path_form();

        print $cgi->end_html();
    }


    exit(0);
}

####
sub print_get_sqlite_db_path_form {
    
    print "<form action='TrinityWeb.cgi' method='get'>\n";

    print "<ul>\n"
        . "<li>Path to Trinotate SQLite database:\n"
        . "<li><input type='text' name='sqlite' maxlength=1000 size=150 />\n"
        . "<li><input type='submit' />\n"
        . "</li>\n"
        . "</ul>\n";
    
    print "</form>\n";


    return;
}
    
####
sub TrinityWebMain {
    my ($params_href, $sqlite_db) = @_;
    
    my $dbproc = DBI->connect( "dbi:SQLite:$sqlite_db" ) || die "Cannot connect: $DBI::errstr";
        
    print "<ul>\n";
    
    print "<li>Stats</li>\n" . &stats_panel_text($dbproc);
    
    print "<li>Search</li>\n" . &search_panel_text($dbproc);
    
    print "<li>Pairwise Expression Comparisons</li>\n" . &pairwise_DE_panel_text($dbproc);

    print "<li>Multi-sample Comparisons</li>\n" . &multi_sample_DE_panel_text($dbproc, $sqlite_db);

    print "</ul>\n";
    

    return;
}

####
sub stats_panel_text {
    my ($dbproc) = @_;
    
    my $html = "<p>Got X transcripts and components</p>\n";
    
    return($html);
    
}

####
sub search_panel_text {
    my ($dbproc) = @_;
    
    my $html = "<p>Search for something</p>\n";

    return($html);
}

####
sub pairwise_DE_panel_text {
    my ($dbproc) = @_;
    
    my $html = "<p>Pairwise matrix of DE transcripts</p>\n";

    return($html);
}

####
sub multi_sample_DE_panel_text {
    my ($dbproc, $sqlite_db) = @_;

    my $html = "<p>Multi-sample differential expression analysis</p>\n";
        
    my $url = "HeatMapNav.cgi?sqlite=" . uri_escape("$sqlite_db");
    $html .= "<p>Go to the <a href=\"$url\">heatmap</a></p>\n"; #?sqlite=$sqlite_db\">heatmap navigator</a></p>\n";
    
    return($html);
}
