#!/usr/bin/env perl

use strict;
use warnings;
use FindBin;

my $usage = <<_EOUSAGE_;

 usage: $0 <command> <input> [...]

 <command>:  LOAD_transdecoder <file>
             LOAD_blast <file>
             LOAD_pfam <file>
             LOAD_tmhmm <file>
             LOAD_signalp <file>
             report [ -E (default: 1e-5) ]


_EOUSAGE_

;

unless (@ARGV) { die $usage; }
if ($ARGV[0] ne "report" && scalar(@ARGV) < 2) { 
    die $usage;
}
elsif ($ARGV[0] eq "report") {
    $ARGV[1] = "";
}

    


my %commands = map { + $_ => 1 } qw(LOAD_transdecoder LOAD_blast LOAD_pfam LOAD_tmhmm LOAD_signalp report);

my $command = $ARGV[0];
unless (exists $commands{$command}) {
    die "$usage\nError, do not recognize command: $command\n";
}



my $CLASSPATH = "$FindBin::Bin/java";
my $java_exe = "java -cp $CLASSPATH:$CLASSPATH/sqlitejdbc-v056.jar";
my $verbosity = "counter";

my %command_to_params = ( 'LOAD_transdecoder' => "TrinityAminoAcidTosqlLite $ARGV[1] $verbosity",
                          'LOAD_blast' => "UniprotBlastoutputtoMySQLLite $ARGV[1] $verbosity uniprot",
                          'LOAD_pfam'  => "HmmerDOMoutMYSQLLite $ARGV[1] $verbosity",
                          'LOAD_tmhmm' => "tmhmmtoMYSQLlite $ARGV[1] $verbosity",
                          'LOAD_signalp' => "SignalPtoMYSQLlite $ARGV[1] $verbosity",
    );



if (my $java_cmd = $command_to_params{$command}) {
    $java_cmd = "$java_exe $java_cmd";
    &process_cmd($java_cmd);
}
elsif ($command eq "report") {
    ## generate Trinotate report.
    shift @ARGV;
    my $cmd = "$FindBin::Bin/util/Trinotate_report_writer.pl @ARGV";
    &process_cmd($cmd);
    
}

exit(0);

####
sub process_cmd {
    my ($cmd) = @_;

    print STDERR "CMD: $cmd\n";
    my $ret = system($cmd);
    if ($ret) {
        die "Error, cmd: $cmd died with ret $ret";
    }

    return;
}
