import java.io.*;
import java.util.StringTokenizer; 
import java.math.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.sql.*;

class CreateJavaSQLliteTables {
    
    public static void main(String args[]) {
        
        try {
            
            
            Class.forName("org.sqlite.JDBC");
            Connection conn = DriverManager.getConnection("jdbc:sqlite:TrinityFunctional.db");
            Statement stat = conn.createStatement();
            
            // UniprotIndex table
            stat.executeUpdate("create table UniprotIndex(Accession,LinkId,AttributeType);");
            stat.executeUpdate("CREATE INDEX UniprotAccession ON UniprotIndex(Accession)");
            
            // PFAM reference database info
            stat.executeUpdate("create table PFAMreference(pfam_accession, pfam_domainname, pfam_domaindescription,"
                               + "Sequence_GatheringCutOff REAL, Domain_GatheringCutOff REAL, Sequence_TrustedCutOff REAL,"
                               + "Domain_TrustedCutOff REAL, Sequence_NoiseCutOff REAL, Domain_NoiseCutOff REAL);");    
            stat.executeUpdate("CREATE UNIQUE INDEX PFAMUniIndex ON PFAMreference(pfam_accession)");
            
            // TaxonomyIndex table 
            stat.executeUpdate("create table TaxonomyIndex(NCBITaxonomyAccession,TaxonomyValue)");
            stat.executeUpdate("CREATE UNIQUE INDEX NCBIUniIndex ON TaxonomyIndex(NCBITaxonomyAccession)");
            
            // BlastDbase table
            stat.executeUpdate("create table BlastDbase(TrinityID,FullAccession,GINumber,UniprotSearchString,QueryStart REAL,QueryEnd REAL,HitStart REAL,HitEnd REAL,PercentIdentity REAL,Evalue REAL,BitScore REAL);");
            stat.executeUpdate("CREATE INDEX TrinityTranscriptID ON BlastDbase(TrinityID)");
            stat.executeUpdate("CREATE INDEX FullAccessionIndex ON BlastDbase(FullAccession)");
            stat.executeUpdate("CREATE INDEX GIIndex ON BlastDbase(GINumber)");
            stat.executeUpdate("CREATE INDEX UniprotSeachStringIndex ON BlastDbase(UniprotSearchString)");
            
            
            // HMMERDbase table
            stat.executeUpdate("create table HMMERDbase(QueryProtID,pfam_id,HMMERDomain,HMMERTDomainDescription,QueryStartAlign REAL,QueryEndAlign REAL,PFAMStartAlign REAL,PFAMEndAlign REAL,FullSeqEvalue REAL,ThisDomainEvalue REAL,FullSeqScore REAL,FullDomainScore REAL);");
            stat.executeUpdate("CREATE INDEX PFAMQueryID ON HMMERDbase(QueryProtID)");
            stat.executeUpdate("CREATE INDEX PFAMDomainID ON HMMERDbase(pfam_id)");
            
            
            // MaxQuantGFFtoTrinity table (not using yet)
            stat.executeUpdate("create table MaxQuantGFFtoTrinity(MaxQuantQueryID,MaxQuantQueryParentID,MaxQuantProteinMatchStart REAL,MaxQuantProteinMatchend REAL,MaxQuantStrand,MaxQuantTrinityMatchID,MaxQuantscoreFKPMValue REAL,MaxQuantPeptideValue REAL,MaxQuantPeptideScore REAL,MaxQuantExonValue REAL);");
            stat.executeUpdate("CREATE INDEX TrinityID ON MaxQuantGFFtoTrinity(MaxQuantQueryID)");;
            
            // SignalP table
            stat.executeUpdate("create table SignalP(query_prot_id,start REAL,end REAL,score REAL,prediction);");
            stat.executeUpdate("CREATE UNIQUE INDEX QueryID ON SignalP(query_prot_id)");
            
            
            // TrinityTranscript table
            stat.executeUpdate("create table TrinityTranscript(TrinityAminoAcidID,PredicitonType,Length REAL,Strand,Path,AminoAcid_sequence);");
            stat.executeUpdate("CREATE UNIQUE INDEX TrinityAminoAcidIDIndex ON TrinityTranscript(TrinityAminoAcidID)");
            
            
            // tmhmm table
            stat.executeUpdate("create table tmhmm(queryprotid,Score REAL,PredHel,Topology);");
            stat.executeUpdate("CREATE UNIQUE INDEX QueryIDtmhmm ON tmhmm(queryprotid)");
            
            
            // eggnog table
            stat.executeUpdate("create table eggNOGIndex(eggNOGIndexTerm,eggNOGDescriptionValue)");
            stat.executeUpdate("CREATE UNIQUE INDEX eggNOGUniIndex ON eggNOGIndex(eggNOGIndexTerm)");
            
            
            // gene ontology
            stat.executeUpdate("CREATE TABLE go (id varchar(20), name TEXT, namespace varchar(30), def TEXT)");
            stat.executeUpdate("CREATE UNIQUE INDEX id_idx ON go(id)");

            //---------------
            // add DE stuff
            //---------------

            // Samples
            stat.executeUpdate("CREATE TABLE Samples (sample_id varchar(4), sample_name)");
            stat.executeUpdate("CREATE UNIQUE INDEX sampleid_idx ON Samples(sample_id)");
            stat.executeUpdate("CREATE UNIQUE INDEX samplename_idx ON Samples(sample_name)");

            // Replicates
            stat.executeUpdate("CREATE TABLE Replicates (replicate_id varchar(4), replicate_name, sample_id)");
            stat.executeUpdate("CREATE UNIQUE INDEX rep_id_idx ON Replicates (replicate_id)");
            stat.executeUpdate("CREATE UNIQUE INDEX rep_name_idx ON Replicates (replicate_name)");
            stat.executeUpdate("CREATE INDEX rep_samp_name_idx ON Replicates (sample_id)");

            // Expression
            stat.executeUpdate("CREATE TABLE Expression (feature_name, feature_type, replicate_id, frag_count REAL, fpkm REAL)");
            stat.executeUpdate("CREATE UNIQUE INDEX feat_replicate_idx ON Expression(feature_name, replicate_id)");
            stat.executeUpdate("CREATE INDEX feat_name_type_idx ON Expression(feature_name, feature_type)");
            stat.executeUpdate("CREATE INDEX feature_name_idx ON Expression(feature_name)");
            
            // Diff Expression
            stat.executeUpdate("CREATE TABLE Diff_expression (sample_id_A, sample_id_B, feature_name, feature_type, log_avg_expr REAL, log_fold_change REAL, p_value REAL, fdr REAL)");
            stat.executeUpdate("CREATE UNIQUE INDEX diff_expr_idx ON Diff_expression (sample_id_A, sample_id_B, feature_name)");
            stat.executeUpdate("CREATE INDEX sample_id_type_idx ON Diff_expression (sample_id_A, sample_id_B, feature_type)");
            stat.executeUpdate("CREATE INDEX diff_expr_feat_name_idx ON Diff_expression(feature_name)");
            
            conn.close();
                        
            
        }
        catch (Exception e){//Catch exception if any
            System.err.println("Error: " + e.getMessage());
        }
    }
}
